
subroutine CalcEnergy(Pos, M, L, rc, NSolvent, NWall, zj, p, PEnergy, Dim, NAtom)
    implicit none
    integer, intent(in) :: M, Dim, NAtom, NSolvent, NWall
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in) :: L, rc, zj, p
    real(8), intent(out) :: PEnergy
    real(8), parameter :: k = 3000., r0 = 1. !, p = 10.
    !======== YOUR CODE HERE ========
    real(8), dimension(Dim) :: rij, Posi
    real(8) :: d2, id2, id6, id12, x, Shift, Shift2, rc2, dis
    integer :: i, j, NSolute
    rc2 = rc * rc
    Shift = -4. * (rc**(-12) - rc**(-6))
    Shift2 = -4. * (rc**(-12) - 2*rc**(-6))
    PEnergy = 0.
    NSolute = NAtom - NWall - NSolvent
    do i = 0, NSolute - 1 !Consider solute PE
        !store Pos(i,:) in a temporary array for faster access in j loop
        Posi = Pos(i,:)
	dis = (Posi(3)-zj)
	dis = dis - L*dnint(dis/L)
	if (zj .ne. 0) then
		PEnergy = PEnergy + p / 2. * (dis)**2 !introduce bias on all solute molecules
	end if 
        do j = i + 1, NSolute - 1 !Solute-solute
            rij = Pos(j,:) - Posi
	    rij = rij - L * dnint(rij / L)
            d2 = sum(rij * rij)
	    if (j == i + 1 .and. mod(j, M) > 0) then 	 !carry out harmonic PE calculation for this i,j
	        x = sqrt(d2)
                PEnergy = PEnergy + k / 2. * (x - r0)**2
	    else 
		if (d2 > rc2) then 
		    cycle
		end if      !carry out LJ PE calculation for this i,j if less than cutoff!
                id2 = 1. / d2            !inverse squared distance
                id6 = id2 * id2 * id2    !inverse sixth distance
                id12 = id6 * id6         !inverse twelveth distance
                PEnergy = PEnergy + 4. * (id12 - 2*id6) + Shift2     !calculated shifted potential
	    end if
        enddo
	do j = NSolute, NSolvent - 1 + NSolute !Solute-solvent
	    rij = Pos(j,:) - Posi
	    rij = rij - L * dnint(rij / L)
            d2 = sum(rij * rij)
	    if (d2 > rc2) then 
		cycle
	    end if      !carry out LJ PE calculation for this i,j if less than cutoff!
            id2 = 1. / d2            !inverse squared distance
            id6 = id2 * id2 * id2    !inverse sixth distance
            id12 = id6 * id6         !inverse twelveth distance
            PEnergy = PEnergy + 4. * (id12 - id6) + Shift     !calculated shifted potential
	enddo
	do j = NSolvent + NSolute, NAtom - 1 !Solute-crystal
	    rij = Pos(j,:) - Posi
	    rij = rij - L * dnint(rij / L)
            d2 = sum(rij * rij)
	    if (d2 > rc2) then 
		cycle
	    end if      !carry out LJ PE calculation for this i,j if less than cutoff!
            id2 = 1. / d2            !inverse squared distance
            id6 = id2 * id2 * id2    !inverse sixth distance
            id12 = id6 * id6         !inverse twelveth distance
            PEnergy = PEnergy + 4. * (id12 - 2*id6) + Shift2     !calculated shifted potential
	enddo
    enddo
    do i = NSolute, NSolvent - 1 + NSolute !Consider Solvent PE
        !store Pos(i,:) in a temporary array for faster access in j loop
        Posi = Pos(i,:)
	do j = i+1, NSolvent - 1 + NSolute!Solvent-solvent
	    rij = Pos(j,:) - Posi
	    rij = rij - L * dnint(rij / L)
            d2 = sum(rij * rij)
	    if (d2 > rc2) then 
		cycle
	    end if      !carry out LJ PE calculation for this i,j if less than cutoff!
            id2 = 1. / d2            !inverse squared distance
            id6 = id2 * id2 * id2    !inverse sixth distance
            id12 = id6 * id6         !inverse twelveth distance
            PEnergy = PEnergy + 4. * (id12 - id6) + Shift     !calculated shifted potential
	enddo
	do j = NSolvent + NSolute, NAtom - 1 !Solvent-crystal
	    rij = Pos(j,:) - Posi
	    rij = rij - L * dnint(rij / L)
            d2 = sum(rij * rij)
	    if (d2 > rc2) then 
		cycle
	    end if      !carry out LJ PE calculation for this i,j if less than cutoff!
            id2 = 1. / d2            !inverse squared distance
            id6 = id2 * id2 * id2    !inverse sixth distance
            id12 = id6 * id6         !inverse twelveth distance
            PEnergy = PEnergy + 4. * (id12 - id6) + Shift     !calculated shifted potential
	enddo
    enddo	
end subroutine


subroutine CalcEnergyForces(Pos, M, L, rc, NSolvent, NWall, zj, p, PEnergy, Forces, Dim, NAtom)
    implicit none
    integer, intent(in) :: M, Dim, NAtom, NSolvent, NWall
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in) :: L, rc, zj, p
    real(8), intent(out) :: PEnergy
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Forces
!f2py intent(in, out) :: Forces
    real(8), parameter :: k = 3000., r0 = 1. !, p = 10.
    !======== YOUR CODE HERE ========    
    real(8), dimension(Dim) :: rij, Posi, Fij
    real(8) :: d2, id2, id6, id12, x, Shift, Shift2, rc2, dis
    integer :: i, j, NSolute
    rc2 = rc * rc
    Shift = -4. * (rc**(-12) - rc**(-6))
    Shift2 = -4. * (rc**(-12) - 2*rc**(-6))
    PEnergy = 0.
    Forces = 0.
    NSolute = NAtom - NWall - NSolvent
    do i = 0, NSolute - 1 !Consider Solute PE and Forces
        !store Pos(i,:) in a temporary array for faster access in j loop
        Posi = Pos(i,:)
	dis = (Posi(3)-zj)
	dis = dis - L*dnint(dis/L)
	if (zj .ne. 0) then
	    PEnergy = PEnergy + p / 2. * (dis)**2 !introduce bias on all solute molecules
            Forces(i,3) = Forces(i,3) - p * (dis)
	end if 
        do j = i + 1, NSolute - 1 !Solute-solute
            rij = Pos(j,:) - Posi
	    rij = rij - L * dnint(rij / L)
            d2 = sum(rij * rij)
	    if (j == i + 1 .and. mod(j, M) > 0) then 	 !carry out harmonic PE calculation for this i,j
	        x = sqrt(d2)
                PEnergy = PEnergy + k / 2. * (x - r0)**2
                Fij = k * (x - r0) * rij / x     !take force as gradient of potential
	    else 
		if (d2 > rc2) then
 		    cycle
 		end if       !carry out LJ PE calculation for this i,j if less than cutoff!
                id2 = 1. / d2            !inverse squared distance
                id6 = id2 * id2 * id2    !inverse sixth distance
                id12 = id6 * id6         !inverse twelveth distance
                PEnergy = PEnergy + 4. * (id12 - 2*id6) + Shift2     !calculated shifted potential
		Fij = rij * ((-48. * id12 + 48. * id6) * id2)
            end if
            Forces(i,:) = Forces(i,:) + Fij
            Forces(j,:) = Forces(j,:) - Fij
        enddo
	do j = NSolute, NSolvent - 1 + NSolute !Solute-solvent
	    rij = Pos(j,:) - Posi
	    rij = rij - L * dnint(rij / L)
            d2 = sum(rij * rij)
	    if (d2 > rc2) then 
		cycle
	    end if      !carry out LJ PE calculation for this i,j if less than cutoff!
            id2 = 1. / d2            !inverse squared distance
            id6 = id2 * id2 * id2    !inverse sixth distance
            id12 = id6 * id6         !inverse twelveth distance
            PEnergy = PEnergy + 4. * (id12 - id6) + Shift     !calculated shifted potential
	    Fij = rij * ((-48. * id12 + 24. * id6) * id2)
	    Forces(i,:) = Forces(i,:) + Fij
            Forces(j,:) = Forces(j,:) - Fij
	enddo
	do j = NSolvent + NSolute, NAtom - 1 !Solute-crystal
	    rij = Pos(j,:) - Posi
	    rij = rij - L * dnint(rij / L)
            d2 = sum(rij * rij)
	    if (d2 > rc2) then 
		cycle
	    end if      !carry out LJ PE calculation for this i,j if less than cutoff!
            id2 = 1. / d2            !inverse squared distance
            id6 = id2 * id2 * id2    !inverse sixth distance
            id12 = id6 * id6         !inverse twelveth distance
            PEnergy = PEnergy + 4. * (id12 - 2*id6) + Shift2     !calculated shifted potential
	    Fij = rij * ((-48. * id12 + 48. * id6) * id2)
	    Forces(i,:) = Forces(i,:) + Fij
            !Forces(j,:) = Forces(j,:) - Fij
	enddo
    enddo
    do i = NSolute, NSolvent - 1 + NSolute !Consider Solvent PE + Forces
        !store Pos(i,:) in a temporary array for faster access in j loop
        Posi = Pos(i,:)
	do j = i+1, NSolvent - 1 + NSolute !Solvent-solvent
	    rij = Pos(j,:) - Posi
	    rij = rij - L * dnint(rij / L)
            d2 = sum(rij * rij)
	    if (d2 > rc2) then 
		cycle
	    end if      !carry out LJ PE calculation for this i,j if less than cutoff!
            id2 = 1. / d2            !inverse squared distance
            id6 = id2 * id2 * id2    !inverse sixth distance
            id12 = id6 * id6         !inverse twelveth distance
            PEnergy = PEnergy + 4. * (id12 - id6) + Shift     !calculated shifted potential
	    Fij = rij * ((-48. * id12 + 24. * id6) * id2)
	    Forces(i,:) = Forces(i,:) + Fij
            Forces(j,:) = Forces(j,:) - Fij
	enddo
	do j = NSolvent + NSolute, NAtom - 1 !Solvent-crystal
	    rij = Pos(j,:) - Posi
	    rij = rij - L * dnint(rij / L)
            d2 = sum(rij * rij)
	    if (d2 > rc2) then 
		cycle
	    end if      !carry out LJ PE calculation for this i,j if less than cutoff!
            id2 = 1. / d2            !inverse squared distance
            id6 = id2 * id2 * id2    !inverse sixth distance
            id12 = id6 * id6         !inverse twelveth distance
            PEnergy = PEnergy + 4. * (id12 - id6) + Shift     !calculated shifted potential
	    Fij = rij * ((-48. * id12 + 24. * id6) * id2)
	    Forces(i,:) = Forces(i,:) + Fij
            !Forces(j,:) = Forces(j,:) - Fij
	enddo
    enddo	
end subroutine

subroutine VVIntegrate(Pos, Vel, Accel, M, L, rc, dt, NSolvent, NWall, zj, p, KEnergy, PEnergy, Dim, NAtom)
    implicit none
    integer, intent(in) :: M, Dim, NAtom, NSolvent, NWall
    real(8), intent(in) :: L, rc, dt, zj, p
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Pos, Vel, Accel
!f2py intent(in,out) :: Pos, Vel, Accel
    real(8), intent(out) :: KEnergy, PEnergy
    external :: CalcEnergyForces
    Pos = Pos + dt * Vel + 0.5 * dt*dt * Accel
    Vel = Vel + 0.5 * dt * Accel
    call CalcEnergyForces(Pos, M, L, rc, NSolvent, NWall, zj, p, PEnergy, Accel, Dim, NAtom)
    Vel = Vel + 0.5 * dt * Accel
    KEnergy = 0.5 * sum(Vel*Vel)
end subroutine